package jalview.datamodel.xdb.embl;

import static org.testng.AssertJUnit.assertEquals;
import static org.testng.AssertJUnit.assertSame;

import jalview.analysis.SequenceIdMatcher;
import jalview.datamodel.DBRefEntry;
import jalview.datamodel.Sequence;
import jalview.datamodel.SequenceI;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.testng.annotations.Test;

public class EmblEntryTest
{
  @Test(groups = "Functional")
  public void testGetCdsRanges()
  {
    EmblEntry testee = new EmblEntry();

    /*
     * Make a (CDS) Feature with 4 locations
     */
    EmblFeature cds = new EmblFeature();
    cds.setLocation("join(10..20,complement(30..40),50..60,70..80,complement(110..120))");

    int[] exons = testee.getCdsRanges(cds);
    assertEquals("[10, 20, 40, 30, 50, 60, 70, 80, 120, 110]",
            Arrays.toString(exons));
  }

  @Test(groups = "Functional")
  public void testParseCodingFeature()
  {
    // not the whole sequence but enough for this test...
    SequenceI dna = new Sequence("J03321", "GGATCCGTAAGTTAGACGAAATT");
    List<SequenceI> peptides = new ArrayList<SequenceI>();
    SequenceIdMatcher matcher = new SequenceIdMatcher(peptides);
    EmblFile ef = EmblTestHelper.getEmblFile();

    /*
     * parse two CDS features, one with two Uniprot cross-refs,
     * the other with one
     */
    EmblEntry testee = new EmblEntry();
    for (EmblFeature feature : ef.getEntries().get(0).getFeatures())
    {
      if ("CDS".equals(feature.getName()))
      {
        testee.parseCodingFeature(feature, "EMBL", dna, peptides, matcher);
      }
    }

    /*
     * peptides should now have five entries:
     * EMBL product and two Uniprot accessions for the first CDS / translation
     * EMBL product and one Uniprot accession for the second CDS / translation
     */
    assertEquals(5, peptides.size());
    assertEquals("CAA30420.1", peptides.get(0).getName());
    assertEquals("MLCF", peptides.get(0).getSequenceAsString());
    assertEquals("UNIPROT|B0BCM4", peptides.get(1).getName());
    assertEquals("MLCF", peptides.get(1).getSequenceAsString());
    assertEquals("UNIPROT|P0CE20", peptides.get(2).getName());
    assertEquals("MLCF", peptides.get(2).getSequenceAsString());
    assertEquals("CAA30421.1", peptides.get(3).getName());
    assertEquals("MSSS", peptides.get(3).getSequenceAsString());
    assertEquals("UNIPROT|B0BCM3", peptides.get(4).getName());
    assertEquals("MSSS", peptides.get(4).getSequenceAsString());

    /*
     * verify dna sequence has dbrefs with mappings to the peptide 'products'
     */
    DBRefEntry[] dbrefs = dna.getDBRefs();
    assertEquals(3, dbrefs.length);
    DBRefEntry dbRefEntry = dbrefs[0];
    assertEquals("UNIPROT", dbRefEntry.getSource());
    assertEquals("B0BCM4", dbRefEntry.getAccessionId());
    assertSame(peptides.get(1), dbRefEntry.getMap().getTo());
    List<int[]> fromRanges = dbRefEntry.getMap().getMap().getFromRanges();
    assertEquals(1, fromRanges.size());
    assertEquals(57, fromRanges.get(0)[0]);
    assertEquals(46, fromRanges.get(0)[1]);
    List<int[]> toRanges = dbRefEntry.getMap().getMap().getToRanges();
    assertEquals(1, toRanges.size());
    assertEquals(1, toRanges.get(0)[0]);
    assertEquals(4, toRanges.get(0)[1]);

    dbRefEntry = dbrefs[1];
    assertEquals("UNIPROT", dbRefEntry.getSource());
    assertEquals("P0CE20", dbRefEntry.getAccessionId());
    assertSame(peptides.get(2), dbRefEntry.getMap().getTo());
    fromRanges = dbRefEntry.getMap().getMap().getFromRanges();
    assertEquals(1, fromRanges.size());
    assertEquals(57, fromRanges.get(0)[0]);
    assertEquals(46, fromRanges.get(0)[1]);
    toRanges = dbRefEntry.getMap().getMap().getToRanges();
    assertEquals(1, toRanges.size());
    assertEquals(1, toRanges.get(0)[0]);
    assertEquals(4, toRanges.get(0)[1]);

    dbRefEntry = dbrefs[2];
    assertEquals("UNIPROT", dbRefEntry.getSource());
    assertEquals("B0BCM3", dbRefEntry.getAccessionId());
    assertSame(peptides.get(4), dbRefEntry.getMap().getTo());
    fromRanges = dbRefEntry.getMap().getMap().getFromRanges();
    assertEquals(1, fromRanges.size());
    assertEquals(4, fromRanges.get(0)[0]);
    assertEquals(15, fromRanges.get(0)[1]);
    toRanges = dbRefEntry.getMap().getMap().getToRanges();
    assertEquals(1, toRanges.size());
    assertEquals(1, toRanges.get(0)[0]);
    assertEquals(4, toRanges.get(0)[1]);
  }
}
